/**
 *  文件下载 - <a />标签方式
 * @param {string}fileUrl :文件url
 * @param {string}fileName :下载后的文件名称（非必填）
 */
function FileDownload(fileUrl: string, fileName?: string) {
  try {
    let link = document.createElement("a");
    if (fileName) link.setAttribute("download", fileName);
    link.href = fileUrl;
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  } catch (e) {
    console.error("文件下载出错了。。。。", e);
  }
}

/**
 *  文件下载 - iframe 方式
 * @param {string}fileUrl :文件url
 */
function IframeDownload(fileUrl: string) {
  try {
    let elemIF = document.createElement("iframe");
    elemIF.src = fileUrl;
    elemIF.style.display = "none";
    document.body.appendChild(elemIF);
  } catch (e) {
    console.error("文件下载出错了。。。。", e);
  }
}

/**
 * 文件下载 - blob 方式
 * @param {string}fileUrl :文件url
 * @param {string}fileName :下载后的文件名称（非必填）
 */
function BlobFileDownload(fileUrl: string, fileName?: string) {
  let xhr = new XMLHttpRequest();
  xhr.open("get", fileUrl, true);
  xhr.responseType = "blob";
  xhr.onload = function () {
    if (this.status == 200) {
      const reader = new FileReader();
      reader.onload = function () {
        let link = document.createElement("a");
        link.setAttribute(
          "href",
          "data:text/plain;charset=utf-8," +
          encodeURIComponent(String(reader.result))
        );
        if (fileName) link.setAttribute("download", fileName);
        document.body.appendChild(link);
        link.click();
        document.body.removeChild(link);
      };
      reader.readAsText(this.response);
    }
  };
  xhr.send();
}

/**
 * 文件下载 - blob 方式 (pdf、mp4 文件流下载)
 * @param {string}fileUrl :文件url
 * @param {string}fileName :下载后的文件名称（非必填）
 */
function BlobDownloadByFileUrl(fileUrl: string, fileName?: string) {
  let xhr = new XMLHttpRequest();
  xhr.open("get", fileUrl, true);
  xhr.responseType = "blob";
  xhr.onload = function () {
    if (this.status == 200) {
      let res = this.response;
      // 创建blob对象，解析流数据
      const blob = new Blob([res], {
        type: res.type,
      });
      const link = document.createElement("a");
      // 兼容webkix浏览器，处理webkit浏览器中href自动添加blob前缀，默认在浏览器打开而不是下载
      const URL = window.URL || window.webkitURL;
      // 根据解析后的blob对象创建URL 对象
      const fileUrl = URL.createObjectURL(blob);
      // 下载文件名,如果后端没有返回，可以自己写a.download = '文件.pdf'
      if (fileName) link.setAttribute("download", fileName);
      // 下载链接
      link.href = fileUrl;
      document.body.appendChild(link);
      link.click();
      document.body.removeChild(link);
      // 在内存中移除URL 对象
      window.URL.revokeObjectURL(fileUrl);
    }
  };
  xhr.send();
}

/**
 * blob 文件下载
 * @param blob 
 * @param fileName 
 */
function BlobDownloadByBlob(blob: Blob, fileName?: string) {
  const objectUrl = URL.createObjectURL(blob);
  const a = document.createElement("a");
  document.body.appendChild(a);
  a.style.display = "none";
  a.href = objectUrl;
  if (fileName) a.download = fileName;
  a.click();
  document.body.removeChild(a);
  // 在内存中移除URL 对象
  // window.URL.revokeObjectURL(blob);
}

/**
 * 获取文件后缀名
 * @param {string}fileUrl 文件地址
 * @returns 文件后缀
 */
function GetFileSuffix(fileUrl: string) {
  if (!fileUrl) return "";
  return fileUrl.slice(fileUrl.lastIndexOf("/") + 1, fileUrl.length);
}

/**
 * 文件类型判断
 * @param {string}fileName
 * @returns 
 */
function FileIconFormat(fileName: string) {
  if (/\.(docx?)$/i.test(fileName)) {
    return "doc.png";
  } else if (/\.(pptx?|ppsx?|ppam?)$/i.test(fileName)) {
    return "ppt.png";
  } else if (/\.(pdf)$/i.test(fileName)) {
    return "pdf.png";
  } else if (/\.(xlsx?)$/i.test(fileName)) {
    return "excel.png";
  } else if (/\.(txt)$/i.test(fileName)) {
    return "txt.png";
  } else if (/\.(psd)$/i.test(fileName)) {
    return "psd.png";
  } else if (/\.(ai)$/i.test(fileName)) {
    return "ai.png";
  } else {
    return "file.png";
  }
}


export {
  FileDownload,
  IframeDownload,
  BlobFileDownload,
  BlobDownloadByFileUrl,
  BlobDownloadByBlob,
  GetFileSuffix,
  FileIconFormat
}
