/***********************策略对象**************************/
let strategies = {
  // 非空校验
  isNonEmpty: function (value: string, errorMsg: string) {
    if (value === '' || value === null || value === undefined) {
      return errorMsg;
    }
  },

  // 非0校验
  isNonZero: function (value: string, errorMsg: string) {
    if (value === "" || value === null || Number(value) === 0) {
      return errorMsg;
    }
  },

  // 最小长度校验
  minLength: function (value: string, length: string, errorMsg: string) {
    if (value.length < Number(length)) {
      return errorMsg;
    }
  },

  // 最大长度校验
  maxLength: function (value: string, length: string, errorMsg: string) {
    if (value.length > Number(length)) {
      return errorMsg;
    }
  },

  // 电话号码校验
  isMobile: function (value: string, errorMsg: string) {
    if (!/^1(3\d|4[5-9]|5[0-35-9]|6[2567]|7[0-8]|8\d|9[0-35-9])\d{8}$/.test(value)) {
      return errorMsg;
    }
  },

  // 数字、字母组合校验
  isCardNo: function (value: string, errorMsg: string) {
    const reg = /(^[a-za-z0-9]+$)/;
    if (!reg.test(value)) {
      return errorMsg;
    }
  },

  // 15、18位身份证校验
  isIDNo: function (value: string, errorMsg: string) {
    const reg = /(^\d{15}$)|(^\d{18}$)|(^\d{17}(\d|X|x)$)/;
    if (!reg.test(value)) {
      return errorMsg;
    }
  },

  // 地址地区校验 'xx省-xx市-xx区'
  areaVerify(val: string) {
    const regex = RegExp('([\u4e00-\u9fa5]+)-([\u4e00-\u9fa5]+)-([\u4e00-\u9fa5]+)');
    return regex.test(val) ? val : "";
  },


  /**
   * 自定义校验 正则表达式
   * @param value 校验的对象
   * @param regExp 校验的正则表达式
   * @param errorMsg 错误提示文案
   * @returns
   */
  isCustomReg: function (value: string, regExp: RegExp, errorMsg: string) {
    console.log('regExp=', regExp);
    if (!regExp.test(value)) {
      return errorMsg;
    }
  },

  /**
   * 自定义校验 方法
   * @param value 校验的对象
   * @param fun 自定义校验函数 返回值:true | false
   * @param errorMsg 错误提示文案
   * @returns
   */
  isCustomFun: function (value: string, fun: Function, errorMsg: string) {
    if (fun && fun(value)) {
      return errorMsg;
    }
  }
};

/***********************Validator 类**************************/
let Validator = function () {
  this.cache = [];
};

/**
  * @param {string}value 校验的对象
  * @param {Object}rules 校验规则 { strategy: string, errorMsg: string, regExp?: RegExp,  length?: number, fun?: Function }
  *                        strategy: 规则名称 必传
  *                        errorMsg: 错误提示文案 必传
  *                        regExp?: 正则表达式对象 【isCustomReg】规则时，必传
  *                        length?: 校验的长度 【minLength、maxLength】规则时，必传
  *                        fun?: 自定义校验函数 【isCustomFun】规则时，必传 返回值 true|false
 */
Validator.prototype.add = function (value: string, rules: any) {
  let self = this;
  for (let i = 0, rule; rule = rules[i++];) {
    (function (rule) {
      let { errorMsg, regExp, strategy, length, fun } = rule;
      self.cache.push(function () {
        if (regExp) return strategies[strategy].apply(value, [value, regExp, errorMsg]);
        if (length) return strategies[strategy].apply(value, [value, length, errorMsg]);
        if (fun) return strategies[strategy].apply(value, [value, fun, errorMsg]);
        return strategies[strategy].apply(value, [value, errorMsg]);
      });
    })(rule)
  }
};

Validator.prototype.start = function () {
  for (let i = 0, validatorFunc; validatorFunc = this.cache[i++];) {
    let errorMsg = validatorFunc();
    if (errorMsg) {
      return errorMsg;
    }
  }
};


/***********************客户调用 示例**************************/
// import Validator from "foundation-utils"
// function requiredVerify(item): String | null {
//   let validator = new Validator();
//   validator.add(item.name, [
//     { strategy: "isNonEmpty", errorMsg: "请输入用户名" },
//     { strategy: "minLength", len: 6, errorMsg: "用户名长度不能小于6位" },
//     { strategy: "maxLength", len: 10, errorMsg: "用户名长度不能大于10位" },
//   ]);
//   validator.add(item.phone, [
//     { strategy: "isNonEmpty", errorMsg: "请输入电话号码" },
//     {
//       strategy: 'isCustomReg',
//       regExp: new RegExp(/^1(3\d|4[5-9]|5[0-35-9]|6[2567]|7[0-8]|8\d|9[0-35-9])\d{8}$/),
//       errorMsg: "请输入11位有效电话号码"
//     }
//   ]);
//   validator.add(item.code, [
//     {
//       strategy: "isCustomFun",
//       fun: (value) => {
//         let reg = /(^[a-za-z0-9]+$)/;
//         return !reg.test(value);
//       },
//       errorMsg: "请输入数组、字母组合的code"
//     },
//   ]);
//   let errorMsg = validator.start();
//   return errorMsg;
// }

export let DemoX1 = function () { };

export { Validator }